from logic import *

AKnight = Symbol("A is a Knight")
AKnave = Symbol("A is a Knave")

BKnight = Symbol("B is a Knight")
BKnave = Symbol("B is a Knave")

CKnight = Symbol("C is a Knight")
CKnave = Symbol("C is a Knave")


# Puzzle 0
# A says "I am both a knight and a knave."
knowledge0 = And(
    
    # A can be either Knight or Knave
    Or(AKnight, AKnave),
    
    Or(
        # Either A's statement is true: He is a Knight and he is a Knave - making him a Knight
        And(AKnight, And(AKnight, AKnave)),
        # OR he is a Knave
        AKnave), 

    # A cannot be both a Knight and a Knave
    Not(And(AKnave, AKnight))
)



# Puzzle 1
# A says "We are both knaves."
# B says nothing.
knowledge1 = And(

    # Both A and B can be either Knight or Knave
    Or(AKnight, AKnave),
    Or(BKnight, BKnave),

    # Both A and B can only be either Knight or Knave but not both
    Not(And(AKnight, AKnave)),
    Not(And(BKnight, BKnave)),

    Or(
        # Either A is a knight and both A and B are Knaves
        And(AKnight, And(AKnave, BKnave)),
        # Or A is a knave and then A and B cannot both be knaves
        And(AKnave, Not(And(AKnave, BKnave)))
    )
)



# Puzzle 2
# A says "We are the same kind."
# B says "We are of different kinds."
knowledge2 = And(

    # Both A and B can be either Knight or Knave
    Or(AKnight, AKnave),
    Or(BKnight, BKnave),

    # Both A and B can only be either Knight or Knave but not both
    Not(And(AKnight, AKnave)),
    Not(And(BKnight, BKnave)),

    Or(
        # Either A is correct (=Knight) and A and B are the same type
        And(AKnight, BKnight),
        # Or B is correct (= Knight) and they are of different kinds
        And(Not(AKnight), BKnight)
    ),
    
    Or(
        # Either B is a knight and they are different
        And(BKnight, Not(AKnight)),
        # Or B is a knave and they are the same
        And(BKnave, AKnave)
    )
)



# Puzzle 3
# A says either "I am a knight." or "I am a knave.", but you don't know which.
# B says "A said 'I am a knave'."
# B says "C is a knave."
# C says "A is a knight."
knowledge3 = And(
    
    # All can be either Knight or Knave
    Or(AKnight, AKnave),
    Or(BKnight, BKnave),
    Or(CKnight, CKnave),

    # None can be both
    Not(And(AKnight, AKnave)),
    Not(And(BKnight, BKnave)),
    Not(And(CKnight, CKnave)),

    Or(
        # Either B is knight, and A is knight, then B is knave
        And(BKnight, AKnight, BKnave),
        # Or B is knight and A is knave, then A would not be a knave
        And(BKnight, Not(AKnight), Not(AKnave)),
        # Or B is a knave
        And(Not(BKnight))
    ),

    Or(
        # Either B's statement is true (= BKnight) and C is a knave
        And(BKnight, CKnave),
        # Or B's statement is not true (= BKnave) and C is not a knave
        And(BKnave, Not(CKnave))
    ),

    Or(
        # Either C's statement is true (= CKnight) and A is a knight
        And(CKnight, AKnight),
        # Or C's statement is not true (= CKnave) and A is a knave
        And(CKnave, Not(AKnight))
    )
)



def main():
    symbols = [AKnight, AKnave, BKnight, BKnave, CKnight, CKnave]
    puzzles = [
        ("Puzzle 0", knowledge0),
        ("Puzzle 1", knowledge1),
        ("Puzzle 2", knowledge2),
        ("Puzzle 3", knowledge3)
    ]
    for puzzle, knowledge in puzzles:
        print(puzzle)
        if len(knowledge.conjuncts) == 0:
            print("    Not yet implemented.")
        else:
            for symbol in symbols:
                if model_check(knowledge, symbol):
                    print(f"    {symbol}")


if __name__ == "__main__":
    main()
